import java.io.*;

public class ClassA implements Serializable {
    String strField;
    int intField;

    /**
     * @serialField strField String null, empty string, or any String value.
     * @serialField intField Integer Any valid integer.
     */
    private static final ObjectStreamField[] serialPersistentFields = {
	new ObjectStreamField("strField", String.class),
	new ObjectStreamField("intField", Integer.TYPE),
    };

    public ClassA(String one, int two) {
	strField = one;
	intField = two;
    }

    private void readObject(ObjectInputStream in) 
	throws IOException, ClassNotFoundException {

	    // Read in serial fields
	    ObjectInputStream.GetField pfields = in.readFields();

	    // Assign serialized values to fields
	    strField = (String)pfields.get("strField", "out-of-luck");
	    if (pfields.defaulted("intField")) {
		System.out.println("intField will use default");
	    }
	    intField = (int)pfields.get("intField", 1000);

	    // Get ObjectStreamClass to get ObjectStreamFields for each field
	    ObjectStreamClass objclass = pfields.getObjectStreamClass();
	    ObjectStreamField[] fields = objclass.getFields();

	    // See what we got
	    for (int i = 0; i < fields.length; i++) {
		print(fields[i]);
	    }
    }

    private void print(ObjectStreamField fd) {
	System.out.println("name: " + fd.getName());
	System.out.println("type: " + fd.getType());
	System.out.println("type code: " + fd.getTypeCode());
	System.out.println("type string: " + fd.getTypeString());
	System.out.println("primitive: " + fd.isPrimitive());
	System.out.println("offset: " + fd.getOffset());
    }

    public String toString() {
	return "strField: " + strField + " intField: " + intField;
    }
}
