import java.lang.ref.*;
import java.util.*;

class Main {
    public static void main(String[] args) {
        C obj = new C();
        ReferenceQueue rq = new ReferenceQueue();

        SoftReference sr = new SoftReference(obj, rq);
        System.out.println(sr.get());                        // C

        WeakReference wr = new WeakReference(obj, rq);
        System.out.println(wr.get());                        // C

        PhantomReference pr = new PhantomReference(obj, rq);
        System.out.println(pr.get());                        // null

        try {
            // Explicitly enqueue the reference and then remove it.
            pr.enqueue();
            System.out.println(pr.isEnqueued());             // true
            Reference r = rq.remove();
            System.out.println(r+": "+r.get());              // Phanthom: null
            r.clear();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        // Make C object no longer strongly-reachable.
        obj = null;

        // Use all available memory to force a garbage-collection.
        eatMemory(new LinkedList());

        try {
            while (true) {
                Reference r = rq.remove();
                if (r != null) {
                    System.out.println(r+": "+r.get());
                
                    if (r instanceof PhantomReference) {
                        r.clear();
                    }
                }
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    static void eatMemory(LinkedList l) {
        try {
            l.add(new double[100000]);
            eatMemory(l);
        } catch (OutOfMemoryError e) {
            System.out.println("**** OUT OF MEMORY ****");
            l.clear();
            System.gc();
        }
    }
}

class C {
    protected void finalize() {
        System.out.println("C.finalize() called.");
    }

    public String toString() {
        return "C";
    }
}

class MemoryPig extends Thread {
    public void run() {
        List l = new LinkedList();

        while (true) {
            try {
                while (true) {
                    l.add(new double[100000]);
                }
            } catch (OutOfMemoryError e) {
                System.out.println("**** OUT OF MEMORY ****");
                l.clear();
                System.gc();
            }
        }
    }
} 
