import java.lang.ref.*;
import java.io.*;
import java.util.*;

class Main {
    public static void main(String[] args) throws IOException {
        WordCount wc = new WordCount(args[0]);
        System.out.println(wc.getMap());

        // Cause an OutOfMemoryException
        eatMemory(new LinkedList());

        // Print out the words again.
        System.out.println(wc.getMap());
    } 

    static void eatMemory(LinkedList l) {
        try {
            l.add(new double[100000]);
            eatMemory(l);
        } catch (OutOfMemoryError e) {
            System.out.println("**** OUT OF MEMORY ****");
            l.clear();
            System.gc();
        }
    }
}

class WordCount {
    String filename;
    SoftReference sr;

    WordCount(String n) throws IOException {
        filename = n;
        sr = new SoftReference(parse(filename));
    }

    Map getMap() throws FileNotFoundException, IOException {
        Map map = (Map)sr.get();

        if (map == null) {
            // Reconstruct the map.
            System.out.println("**** Reconstructing Map ****");
            map = parse(filename);
            sr = new SoftReference(map);
        }
        return map;
    }

    // Returns a map of all the words and their frequency in the file n.
    Map parse(String n) throws FileNotFoundException, IOException {
        Map map = new HashMap();
        BufferedReader rd = new BufferedReader(new FileReader(n));
        String line;

        while ((line = rd.readLine()) != null) {
            StringTokenizer st = new StringTokenizer(line);
            while (st.hasMoreTokens()) {
                String word = st.nextToken();
                Integer ri = (Integer)map.get(word);
                if (ri == null) {
                    map.put(word, new Integer(1));
                } else {
                    map.put(word, new Integer(ri.intValue()+1));
                }
            }
        }
        return map;
    }
}

