import java.net.*;
import java.awt.*;
import java.awt.event.*;

public class SomeAuthenticator extends Authenticator {
    private Frame parent;
    private PasswordAuthentication pw;
    private boolean done = false;

    public SomeAuthenticator(Frame parent) {
        this.parent = parent;
    }

    // Override default to ask user for password
    protected PasswordAuthentication getPasswordAuthentication() {
        pw = null;
        done = false;

        new LoginDialog(parent, 
            getRequestingSite(),
            getRequestingPort(),
            getRequestingProtocol(),
            getRequestingScheme(),
            getRequestingPrompt());

        // Wait until user enters something
        while (!done) {
            try {
                System.out.println("sleeping");
                Thread.sleep(5000);
            } catch (InterruptedException e) {
            } // ignore interrupts
        }
        return pw;
    }

    // Popup Dialog window for user to enter password
    class LoginDialog extends Dialog implements ActionListener {
        private TextField passwd = new TextField("", 5);
        private TextField username = new TextField("", 5);
        private Button ok = new Button("OK");
        private Button dismiss = new Button("DISMISS");

        LoginDialog(Frame f, 
            InetAddress src, int port, String protocol, 
            String scheme, String prompt) {
            super(f, "Login", true);

            setLayout(new GridLayout(0, 2));

            passwd.setEchoChar('*');

            add(new Label("User Name:"));
            add(username);

            add(new Label("Password:"));
            add(passwd);

            if (src != null) {
                add(new Label("Server:"));
                add(new Label(src.getHostName()));

                add(new Label("Port:"));
                add(new Label(port + ""));
            }
            if (protocol != null) {
                add(new Label("Protocol:"));
                add(new Label(protocol));
            }
            if (scheme != null) {
                add(new Label("Authentication:"));
                add(new Label(scheme));
            }
            if (prompt != null) {
                add(new Label("Prompt:"));
                add(new Label(prompt));
            }

            ok.addActionListener(this);
            dismiss.addActionListener(this);

            add(ok);
            add(dismiss);

            addWindowListener(new WindowEventHandler());

            pack();
            show();
        }
    
        public void actionPerformed(ActionEvent evt) {
            if (evt.getSource() == ok) {
                String pwstr = passwd.getText();
                char[] pwbuf = new char[pwstr.length()];
                pwstr.getChars(0, pwbuf.length, pwbuf, 0);

                pw = new PasswordAuthentication(username.getText(), pwbuf);
            }
            done = true;
            dispose();
        }

        class WindowEventHandler extends WindowAdapter {
            public void windowClosing(WindowEvent evt) {
                dispose();
            }
        }
    }

}
