import java.text.AttributedString;
import java.text.AttributedCharacterIterator;
import java.awt.im.InputMethodHighlight;
import java.awt.font.TextAttribute;
import java.util.HashMap;
import java.util.Locale;

class Main {
    public static void main(String[] args) {
        // Create a large attributed string. (OK, it's not very large)
        AttributedString largeAS = new AttributedString(
                  "The quick brown fox jumped over the lazy dog.");

        // Add LANGUAGE attribute key/value pair to entire attribute string.
        largeAS.addAttribute(AttributedCharacterIterator.Attribute.LANGUAGE, 
            Locale.ENGLISH);

        // Add POSTURE attribute key/value pair to part of attribute string.
        largeAS.addAttribute(TextAttribute.POSTURE, 
                             TextAttribute.POSTURE_OBLIQUE, 10, 19);

        // Get the iterator for the large string and print the string.
        AttributedCharacterIterator largeACI = largeAS.getIterator();
        System.out.println("Large string:");
        printText(largeACI);

        // Print the attribute key/value pairs for the large string.
        System.out.println("\nAttribute key/values in large string:");
        printAttrs(largeACI, 15);
        System.out.println();

        // Create an array of attribute keys for matching.
        TextAttribute[] matchingAttrs = new TextAttribute[]{
            TextAttribute.POSTURE};

        // Create an attributed substring from the larger string.
        AttributedString smallAS = 
            new AttributedString(largeACI, 10, 19, matchingAttrs);

        // Get iterator for the small string and print the string.
        AttributedCharacterIterator smallACI = smallAS.getIterator();
        System.out.println("Small string:");
        printText(smallACI);

        // Print the attribute key/value pairs for the small string.
        System.out.println("\nAttribute key/values in small string:");
        printAttrs(smallACI, 5);
    }

    public static void printText(AttributedCharacterIterator aci) {
        // Print the entire text using the character iterator
        while (aci.current() != AttributedCharacterIterator.DONE) {
            System.out.print(aci.current());
            aci.next();
        }
        System.out.println("");
    }

    // Print all keys and the value at a particular character position.
    public static void printAttrs(AttributedCharacterIterator aci, int pos) {
        aci.setIndex(pos);
        Object[] attrArray = aci.getAllAttributeKeys().toArray();
        for(int i=0; i<attrArray.length; i++) {
            System.out.print(attrArray[i] + "     ");
            System.out.println(aci.getAttribute(
                (AttributedCharacterIterator.Attribute)attrArray[i]));
        }
    }
}
