import java.util.*;

class Main {
    public static void main(String[] args) {
        List list = new FixedList(2);
        list.add("dog");
        list.add("cat");
        //list.add("pig");             // IllegalStateException
        list.remove("dog");
        list.add("pig");
        System.out.println( list );    // [cat, pig]

        Iterator it = list.iterator();
        list.remove("cat");
        //it.next();                   // ConcurrentModificationException
    }
}

class FixedList extends AbstractList {
    int size;
    Object[] array;

    FixedList(int capacity) {
        array = new Object[capacity];
    }
    
    FixedList() {
        this(10);  // Default capacity is 10
    }

    FixedList(Collection c) {
        int cSize = c.size();
        array = new Object[(cSize*110)/100];  // Allow 10% for growth
        c.toArray(array);
        size = cSize;
    }

    public void add(int index, Object e) {
        if (index < 0 || index > size) {
            throw new IndexOutOfBoundsException();
        }
        if (size==array.length) {
            throw new IllegalStateException();
        }
        System.arraycopy(array, index, array, index+1, size-index);
        array[index] = e;
        size++;
        modCount++;
    }
    public Object get(int index) {
        checkIndex(index);
        return array[index];
    }

    public Object remove(int index) {
        checkIndex(index);
        Object e = array[index];
        System.arraycopy(array, index+1, array, index, size-index-1);
        size--;
        array[size] = null;  // Eliminate "false reference"
        modCount++;
        return e;
    }

    // This method is not strictly necessary, but it speeds up the
    // clear method on this list and its sublists.
    protected void removeRange(int fromIndex, int toIndex) {
        // Needn't check indices, as this is only used internally
        int numMoved = size - toIndex;
        System.arraycopy(array, toIndex, array, fromIndex, numMoved);

        //  Eliminate "false references" to help garbage collector
        int newSize = size - (toIndex-fromIndex);
        while (size != newSize)
            array[--size] = null;
    }

    public Object set(int index, Object e) {
        checkIndex(index);
        Object o = array[index];
        array[index] = e;
        return o;
    }
        
    public int size() {
        return size;
    }

    // Utility method.
    private void checkIndex(int index) {
        if (index < 0 || index >= size) {
            throw new IndexOutOfBoundsException();
        }
    }
}
