import java.util.*;

class Main {
    public static void main(String[] args) {
        Map map = new FixedMap(2);
        map.put("dog", "spot");
        map.put("cat", "luck");
        //map.put("pig", "babe");         // IllegalStateException

        System.out.println(map.get("dog"));
        System.out.println(map.put("dog", "ruff"));
        System.out.println(map.remove("cat"));
        System.out.println(map);

        System.out.println(map.put("cat", "luck"));
        System.out.println(map.keySet());
        System.out.println(map.values());

        System.out.println(map.keySet().remove("dog"));
        System.out.println(map);
        System.out.println(map.values().remove("luck"));
        System.out.println(map);
    }
}

class FixedMap extends AbstractMap {
    private Entry[] entry;
    int size = 0;

    FixedMap(int capacity) {
        entry = new Entry[capacity];
        for (int i=0; i<capacity; i++)
            entry[i] = new Entry();
    }

    FixedMap() {
        this(10);  // Default capacity is 10
    }

    FixedMap(Map m) {
        entry = new Entry[(m.size()*110)/100];  // Allow >= 10% for growth
        putAll(m);
    }

    public Object put(Object key, Object value) {
        for (int i=0; i<size; i++) {
            if (eq(entry[i].key, key)) {
                Object oldVal = entry[i].value;
                entry[i].value = value;
                return oldVal;
            }
        }
        if (size == entry.length)
            throw new IllegalStateException();
        entry[size].key = key;
        entry[size++].value = value;
        return null;
    }

    public Set entrySet() {
        return new AbstractSet() {
            public int size() {
                return size;
            }
            public Iterator iterator() {
                return new Iterator() {
                    private int cursor = 0;
                    private boolean removeProhibited = true;
                    public boolean hasNext() {
                        return cursor < size;
                    }
                    public Object next() {
                        if (cursor==size)
                            throw new NoSuchElementException();
                        removeProhibited = false;
                        return entry[cursor++];
                    }
                    public void remove() {
                        if (removeProhibited)
                            throw new IllegalStateException();
                        entry[--cursor].key = entry[--size].key;
                        entry[cursor].value = entry[size].value;
                        entry[size].key = entry[size].value = null;
                        removeProhibited = true;
                    }
                };
            }
        };
    }

    private static class Entry implements Map.Entry {
        Object key, value;
        public Object getKey() {
            return key;
        }
        public Object getValue() {
            return value;
        }
        public Object setValue(Object v) {
            Object oldVal = value;
            value = v;
            return oldVal;
        }
        public boolean equals(Object o) {
            if (!(o instanceof Map.Entry))
                return false;
            Map.Entry e = (Map.Entry) o;
            return eq(e.getKey(), key) && eq(e.getValue(), value);
        }
        public int hashCode() {
            return hash(key) ^ hash(value);
        }
        public String toString() {
            return key + "=" + value;
        }
    }

    private static boolean eq(Object o1, Object o2) {
        return (o1==null ? o2==null : o1.equals(o2));
    }
    private static int hash(Object o) {
        return (o==null ? 0 : o.hashCode());
    }
}
