import java.util.*;
import java.lang.reflect.*;

class Main {
    public static void main(String[] args) {
        // Create a collection of strings.
        Set col = new HashSet();
        col.add("apple");
        col.add("orange");
        col.add("banana");

        Collection set = new FixedSet(10);

        // Create two objects that are equal.
        String s1 = new String("apple");
        String s2 = new String("apple");
        System.out.println(s1 == s2);          // false

        // Add one of the strings.
        System.out.println( set.add(s1) );     // true
        System.out.println( set.size() );      // 1

        // Add another string.
        System.out.println( set.add(s2) );     // false
        System.out.println( set.size() );      // 1

        // Now add a collection.
        set.addAll(col);
        System.out.println( set );             // apple orange banana

        // Compare with another collection.
        System.out.println( set.equals(col) ); // true

        System.out.println( set.add("grape") );

        // Remove some elements.
        System.out.println( set.remove("lemon") ); // false
        System.out.println( set.remove("apple") ); // true

        // Compare with another collection.
        System.out.println( set.equals(col) ); // false

        // Retain all 
        System.out.println( set.retainAll(col) );

        String[] strings = (String[])set.toArray(new String[0]);
        for (int i=0; i<strings.length; i++) {
            System.out.println(strings[i]);
        }
    }
}

class FixedSet implements Collection {
    Object[] array;

    FixedSet(int elements) {
        array = new Object[elements];
    }
    
    public boolean add(Object o) {
        if (contains(o)) {
            return false;
        }
        for (int i=0; i<array.length; i++) {
            if (array[i] == null) {
                array[i] = o;
                return true;
            }
        }
        throw new IllegalArgumentException();
    }

    public boolean addAll(Collection c) {
        boolean modified = false;

        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
            modified |= add(iter.next());
        }
        return modified;
    }

    public void clear() {
        for (int i=0; i<array.length; i++) {
            array[i] = null;
        }
    }

    public boolean contains(Object o) {
        if (o == null) {
            throw new IllegalArgumentException();
        }
        for (int i=0; i<array.length; i++) {
            if (o.equals(array[i])) {
                return true;
            }
        }
        return false;
    }

    public boolean containsAll(Collection c) {
        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
            if (!c.contains(iter.next())) {
                return false;
            }
        }
        return true;
    }

    public boolean equals(Object o) {
        return o instanceof Set && ((Set)o).size() == size() 
            && containsAll((Set)o);
    }

    public int hashCode() {
        int result = 0;
        for (int i=0; i<array.length; i++) {
            if (array[i] != null) {
                result += array[i].hashCode();
            }
        }
        return result;
    }

    public boolean isEmpty() {
        return size() == 0;
    }

    public Iterator iterator() {
        return new Iterator() {
            private boolean hasNext = true;
            public boolean hasNext() {
                return hasNext;
            }
            public Object next() {
                if (hasNext) {
                    hasNext = false;
                    return null;
                }
                throw new NoSuchElementException();
            }
            public void remove() {
                throw new UnsupportedOperationException();
            }
        };
    }

    public int size() {
        int count = 0;
        for (int i=0; i<array.length; i++) {
            if (array[i] != null) {
                count++;
            }
        }
        return count;
    }

    public boolean remove(Object o) {
        if (o == null) {
            throw new IllegalArgumentException();
        }
        for (int i=0; i<array.length; i++) {
            if (o.equals(array[i])) {
                array[i] = null;
                return true;
            }
        }
        return false;
    }

    public boolean retainAll(Collection c) {
        boolean modified = false;

        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
            modified |= remove(iter.next());
        }
        return modified;
    }

    public boolean removeAll(Collection c) {
        boolean modified = false;

        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
            modified |= remove(iter.next());
        }
        return modified;
    }

    public Object[] toArray() {
        return toArray(new Object[0]);
    }

    public Object[] toArray(Object[] a) {
        int len = size();
        if (a.length < len) {
            a = (Object[])Array.newInstance(
                a.getClass().getComponentType(), len);
        }

        Iterator it = iterator();
        for (int i=0; i<len; i++) {
            a[i] = it.next();
        }

        // Set the last element to null.
        if (len < a.length) {
            a[len] = null;
        }
        return a;
    }

    public String toString() {
        String result = "";

        for (int i=0; i<array.length; i++) {
            if (array[i] != null) {
                if (result.length() > 0) {
                    result += " ";
                }
                result += array[i];
            }
        }
        return result;
    }
}
